from .account.role import RoleSyncService
from .account.user import UserSyncService
from .organization.fixture import FixtureSyncService, FixtureTeamSyncService
from .organization.league import LeagueSyncService
from .organization.manager import ManagerSyncService
from .organization.player import PlayerSyncService
from .organization.referee import RefereeSyncService
from .organization.schedule import ScheduleSyncService
from .organization.season import SeasonSyncService
from .organization.standings import StandingsSyncService
from .organization.team import TeamSyncService
from .organization.org import OrganizationSyncService

__all__ = [
    "RoleSyncService",
    "UserSyncService",
    "FixtureSyncService",
    "FixtureTeamSyncService",
    "LeagueSyncService",
    "ManagerSyncService",
    "PlayerSyncService",
    "RefereeSyncService",
    "ScheduleSyncService",
    "SeasonSyncService",
    "StandingsSyncService",
    "TeamSyncService",
    "OrganizationSyncService",
]



from .account.role import RoleAsyncService
from .account.user import UserAsyncService
from .organization.fixture import FixtureAsyncService, FixtureTeamAsyncService
from .organization.league import LeagueAsyncService
from .organization.manager import ManagerAsyncService
from .organization.player import PlayerAsyncService
from .organization.referee import RefereeAsyncService
from .organization.schedule import ScheduleAsyncService
from .organization.season import SeasonAsyncService
from .organization.standings import StandingsAsyncService
from .organization.team import TeamAsyncService

__all__ = [
    "RoleAsyncService",
    "UserAsyncService",
    "FixtureAsyncService",
    "FixtureTeamAsyncService",
    "LeagueAsyncService",
    "ManagerAsyncService",
    "PlayerAsyncService",
    "RefereeAsyncService",
    "ScheduleAsyncService",
    "SeasonAsyncService",
    "StandingsAsyncService",
    "TeamAsyncService",
]



from advanced_alchemy.filters import FilterTypes

from leaguemanager.models import Manager
from leaguemanager.repository import ManagerSyncRepository
from leaguemanager.repository._async import ManagerAsyncRepository
from leaguemanager.services.base import SQLAlchemyAsyncRepositoryService, SQLAlchemySyncRepositoryService

__all__ = ["ManagerSyncService", "ManagerAsyncService"]


class ManagerSyncService(SQLAlchemySyncRepositoryService):
    """Handles sync database operations for managers."""

    repository_type = ManagerSyncRepository


class ManagerAsyncService(SQLAlchemyAsyncRepositoryService):
    """Handles database operations for managers."""

    repository_type = ManagerAsyncRepository



from datetime import UTC, datetime
from uuid import UUID

from attrs import define, field, validators
from sqlalchemy import UUID as SA_UUID
from sqlalchemy import Boolean, Column, DateTime, ForeignKey, Integer, String, Table, UniqueConstraint
from sqlalchemy.ext.associationproxy import AssociationProxy, association_proxy
from sqlalchemy.orm import relationship
from typing_extensions import TYPE_CHECKING

from leaguemanager.models.base import UUIDAuditBase, mapper, metadata
from leaguemanager.models.enums import Category, Division, MatchDay

if TYPE_CHECKING:
    from ..account.address import Address
    from .league import League
    from .season import Season


@define(slots=False)
class Organization(UUIDAuditBase):
    """Defines an organization with certain rules, as well as divisions and categories."""

    name: str | None = field(default=None, validator=validators.max_len(80))
    description: str | None = field(default=None, validator=validators.optional(validators.max_len(255)))


organization = Table(
    "organization",
    metadata,
    Column("id", SA_UUID, primary_key=True),
    Column("name", String(80), nullable=False, unique=True),
    Column("description", String(255), nullable=True),
    Column("created_at", DateTime, default=lambda: datetime.now(UTC)),
    Column("updated_at", DateTime, default=lambda: datetime.now(UTC), onupdate=datetime.now(UTC)),
)

# ORM Relationships
mapper.map_imperatively(
    Organization,
    organization,
    properties={
        "seasons": relationship("Season", back_populates="organization"),
        "address": relationship("Address", back_populates="organization"),
    },
)



from advanced_alchemy.filters import FilterTypes

from leaguemanager.models import Organization
from leaguemanager.repository import OrganizationSyncRepository
from leaguemanager.repository._async import OrganizationAsyncRepository
from leaguemanager.services.base import SQLAlchemyAsyncRepositoryService, SQLAlchemySyncRepositoryService

__all__ = ["OrganizationSyncService", "OrganizationAsyncService"]


class OrganizationSyncService(SQLAlchemySyncRepositoryService):
    """Handles sync database operations for the organization."""

    repository_type = OrganizationSyncRepository


class OrganizationAsyncService(SQLAlchemyAsyncRepositoryService):
    """Handles async database operations for the organization."""

    repository_type = OrganizationAsyncRepository



from ._sync import (
    AddressSyncRepository,
    CitySyncRepository,
    CountrySyncRepository,
    FixtureSyncRepository,
    FixtureTeamSyncRepository,
    LeagueSyncRepository,
    ManagerSyncRepository,
    OrganizationSyncRepository,
    PlayerSyncRepository,
    RefereeSyncRepository,
    RoleSyncRepository,
    RulesetSyncRepository,
    ScheduleSyncRepository,
    SeasonSyncRepository,
    StandingsSyncRepository,
    StateSyncRepository,
    TeamSyncRepository,
    UserRoleSyncRepository,
    UserSyncRepository,
)

__all__ = [
    "AddressSyncRepository",
    "CitySyncRepository",
    "CountrySyncRepository",
    "FixtureSyncRepository",
    "FixtureTeamSyncRepository",
    "LeagueSyncRepository",
    "ManagerSyncRepository",
    "OrganizationSyncRepository",
    "PlayerSyncRepository",
    "RefereeSyncRepository",
    "RoleSyncRepository",
    "RulesetSyncRepository",
    "ScheduleSyncRepository",
    "SeasonSyncRepository",
    "StandingsSyncRepository",
    "StateSyncRepository",
    "TeamSyncRepository",
    "UserRoleSyncRepository",
    "UserSyncRepository",
]



from datetime import UTC, datetime
from uuid import UUID

from attrs import define, field, validators
from sqlalchemy import UUID as SA_UUID
from sqlalchemy import Boolean, Column, DateTime, ForeignKey, Integer, String, Table, UniqueConstraint
from sqlalchemy.ext.associationproxy import AssociationProxy, association_proxy
from sqlalchemy.orm import relationship
from typing_extensions import TYPE_CHECKING

from leaguemanager.models.base import UUIDAuditBase, mapper, metadata
from leaguemanager.models.enums import Category, Division, MatchDay

if TYPE_CHECKING:
    from .schedule import Schedule
    from .season import Season
    from .team import Team


@define(slots=False)
class League(UUIDAuditBase):
    """Defines a Generic League linked to a specific Ruleset."""

    season_id: UUID | None = field(default=None)

    active: bool = field(default=True)

    name: str | None = field(default=None, validator=validators.max_len(80))
    description: str | None = field(default=None, validator=validators.optional(validators.max_len(255)))

    sport: str | None = field(default=None, validator=validators.optional(validators.max_len(80)))


# SQLAlchemy Imperative Mapping

league = Table(
    "league",
    metadata,
    Column("id", SA_UUID, primary_key=True),
    Column("season_id", SA_UUID, ForeignKey("season.id")),
    Column("active", Boolean, default=True),
    Column("name", String(80), nullable=False, unique=True),
    Column("description", String(255), nullable=True),
    Column("sport", String(80), nullable=True),
    Column("created_at", DateTime, default=lambda: datetime.now(UTC)),
    Column("updated_at", DateTime, default=lambda: datetime.now(UTC), onupdate=datetime.now(UTC)),
)

# ORM Relationships

mapper.map_imperatively(
    League,
    league,
    properties={
        "ruleset": relationship(
            "Ruleset",
            back_populates="leagues",
            uselist=False,
            lazy="selectin",
        ),
        "season": relationship("Season", back_populates="leagues", lazy="selectin"),
        "schedule": relationship("Schedule", back_populates="league", uselist=False, lazy="selectin"),
        "teams": relationship("Team", back_populates="league", lazy="selectin"),
    },
)
